#ifndef NANITLIB_H_
#define NANITLIB_H_

#define NANIT_MAJOR_VERSION (1) ///< Головна версія бібліотеки
#define NANIT_MINOR_VERSION (3) ///< Мінорна версія бібліотеки
#define NANIT_PATHC_VERSION (3) ///< Pathc версія бібілотеки

#define NANIT_SERIAL_SPEED (9600) ///< Швидкість серійного порту

#include "Version.hpp"
#include "Approof/Approof.hpp"

#ifdef __AVR_ATmega2560__

#define BATTARY_PIN (69) ///< Пін підключення батареї вхід АЦП для перевірки стану батареї
#define BUILDIN_STRIP_LED (27) ///< Пін підключення вбудованого адресного світлодіоду

// /**
//  * Піни які відрізняються в різних версіях плати
//  */

#define MOTOR1_A ((getBoardVersion() >= Version(3, 1)) ? (7) : (10)) 
#define MOTOR1_B ((getBoardVersion() >= Version(3, 1)) ? (8) : (9))
#define P1_4 ((getBoardVersion() >= Version(3, 1)) ? (7) : (10))
#define P1_3 ((getBoardVersion() >= Version(3, 1)) ? (8) : (9))
#define P8_3 ((getBoardVersion() >= Version(3, 1)) ? (5) : (54))
#define P10_3 ((getBoardVersion() >= Version(3, 1)) ? (24) : (5))
#define P10_2 ((getBoardVersion() >= Version(3, 1)) ? (54) : (24))
#define P7_4 ((getBoardVersion() >= Version(3, 1)) ? (9) : (7))
#define P9_4 ((getBoardVersion() >= Version(3, 1)) ? (10) : (8))

//запустить драйвер на мотор
#define MOTOR_ENABLE                                                           \
  (40) ///< Пін запуску драйверу моторів
       ///
       /// для дозволу обертання двигунів цей пін має мати високий рівень
       /// доступний лише для запису
#define MOTOR_FAIL (41) ///< Перевірка стану драйверу моторів
/// Доступний лише для читання

#define MOTOR2_A (11)
#define MOTOR2_B (12)

#define BUTTON (3) ///< Сенсорна кнопка підключена до роз'єму \b J3
#define J_7 (18) ///< Роз'єм на платі \b J7  

///Порт 1
#define P1_2 (A6) 
#define P1_1 (A7) 

///Порт 2
// #define P2_6 A8  
#define P2_4 (A8)
#define P2_3 (42) 
#define P2_2 (A9)
#define P2_1 (A10)

///Порт 3
// #define P3_6 31 /// Стара версія
#define P3_4 (31)
#define P3_3 (30)
#define P3_2 (25)
#define P3_1 (71) ///< Пін визначений у розширеному ядрі /bMEGACORE
#ifdef MEGACORE
/// У стандарному інтерфейсі Arduino цей пін не доступний. На платах Arduino де
/// використовується мікроконтролер ATMega2560 цей виввід контролера не
/// розведений на платі. На платах Nanit цей пін розведений та доступний для
/// використання. Щоб у повній мірі використовувати всі можлтвості плати
/// потрібно встановити мануально розширене ядро /bMEGACORE, або скористатись менеджером плат

#else
  #warning Install Nanit board manager 
  #warning https://nanitrobot.github.io/package_NanitRobot_index.json
#endif

//Порт 4
// #define P4_6 (46) // ШІМ, червоний світлодіод (стара версія)
#define P4_4 (46) // ШІМ, червоний світлодіод
#define P4_3 (45) // ШІМ, синій світлодіод
#define P4_2 (44) // ШІМ, зелений світлодіод
#define P4_1 (A11)

///Порт 5
// #define P5_6 33 /// Стара версія
#define P5_4 (33) 
#define P5_3 (A12)
#define P5_2 (A1)
#define P5_1 (2) // INT.0 pin

///Порт 6
// #define P6_6 22 /// Стара версія
#define P6_4 (22)
#define P6_3 (A13)
#define P6_2 (A14)
#define P6_1 (23)

///Порт 7
#define P7_3 (28)
#define P7_2 (A4)
#define P7_1 (A3)

///Порт 8


///Порт 9
#define P9_3 (19)  /// INT.2 pin
#define P9_2 (20) ///< SDA
#define P9_1 (21) ///< SCL

///Порт 10
// #define P10_6 (6)  /// Стара версія
#define P10_4 (6)
#define P10_1 (A2)

///Порт 11
// #define P11_6 (32)   /// Стара версія
#define P11_4 (32)
#define P11_3 (43)

///Порт 12
// #define P12_6 (11)/// Мотор М2_А /// Стара версія
#define P12_4 (11)
#define P12_3 (12)/// Мотор М2_В
#define P12_2 (34)
#define P12_1 (36)

//порти дисплею

#define TFT_CS (48) 
#define TFT_RES (37)
#define TFT_DC (49)
#define TFT_BL (4)
#define TFT_SCK (52)
#define TFT_MOSI (51)



// Контроль заряду батареї
// Опорна напруга АЦП
#define AVCC_REF (5.f) ///< Опорна напруга для вбулованого АЦП (5 вольт)
// Розрядність АЦП
#define ADC_BITRATE (10) ///< Визначення розміру бітрейту АЦП перетворювача (10 розрядний перетворювач)
#define BAT_FULL_CHARGE                                                        \
  (4.19f) ///< Визначення рівня повного рівня заряду викорисатної батареї а
          ///< блоці. Змінна введена на випадок, якщо в подальшому модернізації
          ///< плати буде замінено тип батареї
// Яка батарея використовується
#define LI_ION ///< Визначає тип використаної батареї
/// Визначення введене для випадок зміни типу батареї в процесі розвитку проекту

#elif 0 //
// тут будуть знаходитись визначення якщо в процесі розвитку проекту буде замінено мікроконтролер
// щоб зберігти сумісність з версіями плати на ATmega2560
#else
// Повідомлення що потрібно використовувати розширене ядро Arduino MEGACORE
#warning Check the project settings. Invalid microcontroller selected. This project cannot be built for this processor
#endif

#include "DependsLib.h" ///< Залежності від бібліотек третіх сторін

#include "NanitInits.h"

#include "SerialNumber.hpp" /// < Контроль версії бібліотеки та плати під час виконання коду

/**
 * @brief Отримати версію плати
 */

Version getBoardVersion();

/**
 * @brief Отримати версію бібліотеки
*/
Version getLibVersion();

/**
 * @brief Отримати серійний номер плати
 * 
 * Серійний номер плати читається з вбудованої пам'яті
*/
String getSerialNumber();

/**
 * Вивести на дисплей інформацію про \b Nanit
*/
void NanitInfo();

/**
 * Ініціація змінних та дисплею
*/
void Nanit_Base_Start();

void Nanit_RGB_Write(byte red, byte green, byte blue);
void Nanit_Servo_Rotate(byte angle);

void Nanit_ActiveBuzz_Scream(byte times, int duration);

bool Nanit_Sound_IsSoundDetected(int sound_limit);

#define START_NANIT ::NanitRobot::Nanit::getNanit()
#define GET_NANIT ::NanitRobot::Nanit::getNanit()

#define NANIT_BEGIN(X)                                                   \
  ::NanitRobot::Nanit &X { START_NANIT }

namespace NanitRobot {
/**
 * @brief Мапінг функція для чисел з плаваючою крапкою
 * 
 * @param inputValue[in] значення яке потрібно промаштабувати
 * @param inputMin[in] Нижній поріг значань які потрібно промаштабувати
 * @param inputMax[in] Верхній поріг значень, які потрібно промаштвабувати
 * @param rangeMin[in] Нижня межа значень діапазону вихідних значень
 * @param rangeMax[in] Верхня межа значень діапазону вихідних значень
 * @return повертає маштабоване значення
*/  
float Map(float inputValue, float inputMin, float inputMax, float rangeMin,
          float rangeMax);
          
class Nanit {
public:

enum class GuageType{SmileBatt,Volts,Percent,LAST};
  /**
   * Отримати адресу розміщення створеного об'єкту класу Nanit
  */
  inline static Nanit &getNanit() {
    static Nanit instance;
    return instance;
  }
  /**
   * @brief Get the Batary Voltage object
   * 
   * @return float 
   */
  float getBataryVoltage() const;
  /**
   * @brief Get the Battary Power object
   * 
   * @return float 
   */
  float getBattaryPower() const;
  /**
   * @brief 
   * 
   * @param void 
   */
  void DrawBattGuage(GuageType type= GuageType::SmileBatt)const;
  // L298NX2 DCMotors(MOTOR_ENABLE, MOTOR1_A, MOTOR2_A, MOTOR_ENABLE, MOTOR1_B, MOTOR2_B);
  // 1 & 12 motors
  // class DCMotor {
  // public:
  //   DCMotor() {}
  //   void Begin() {}
  //   void End() {}
  //   void Roll() {}
  //   void Brake() {}
  //   void Forward() {}
  //   void Reverse() {}
  //   void Run(signed short speed1, signed short speed2) {}
  // } Motors;
  // // 2 & 11
  // class Servos {
  // } Servo;
  // 3 color & PIR
  // 4 RGB
		// 4 RGB 
  // 4 RGB
  // 5 Buzzer & mic
  // 6 ultrasonic
  // 7 joy
  // 8 interfase
  // 9 Gyro
  // 10 line

  void BuildinRGB(uint8_t red, uint8_t grean, uint8_t blue,
                  uint8_t brightness = 255) {
    _strip_led.setBrightness(brightness);
    _strip_led.setPixelColor(0, _strip_led.Color(red, grean, blue));
    _strip_led.show();
  }
  void BuildinRGB(uint32_t color, uint8_t brightness = 255) {
    _strip_led.setPixelColor(0, color);
    _strip_led.show();
  }
  Servo ServoMotor;

private:
  /**
   * @brief Об'єк ткласу Nanit є об'єктом одинаком.
   * 
   * Якщо об'єкт не був створений у оперативній пам'яті ми його створюєм. Якщо об'єкт вже існував в пам'яті то ми отримаємо адресу розміщення.
   * Такий спосіб не дозволяє не обмежено створювати об'єкти і дозволяє лише один раз ініціювати необхідні змінні для подальшого використання
  */
  inline Nanit()
      : Display{Adafruit_ST7735(TFT_CS, TFT_DC, TFT_RES)}
//  ,        BoardVersion{getBoardVersion()}
  {
    if(!Serial)
    Serial.begin(NANIT_SERIAL_SPEED);
    Serial.println("Nanit initialise...");
    _strip_led.begin();

    Display.initR(INITR_BLACKTAB);

    Display.fillScreen(ST7735_WHITE);
    Display.setTextColor(ST7735_BLACK);

    pinMode(TFT_BL, OUTPUT);
    digitalWrite(TFT_BL, HIGH);
    Display.setRotation(1);
    {
      pinMode(J_7, INPUT_PULLUP);
      if (!digitalRead(J_7)) {
        Display.println();
        Display.println(" Port diagnostics mode.");
        Display.println();
        Display.println(" You should have a probe");
        Display.println("     to connect to the  ");
        Display.println("     ports              ");
        PinTestSetup();
        while (!digitalRead(J_7))
          PinTestLoop();
      }
    }

    // APROOF_MODE;
  };
  /**
   * @brief Destroy the Nanit object
   * 
   */
  inline ~Nanit()=default;
  FastLED_NeoPixel<1, BUILDIN_STRIP_LED, NEO_GRB> _strip_led;
  public:
  Adafruit_ST7735 Display;
};
}// namespace NanitRobot

#endif

